@@ -690,26 +690,25 @@ def executeJSONRPC(jsonrpccommand):  # NOSONAR
         pass
 
     json_responses = os.environ.get("KODI_STUB_RPC_RESPONSES")
-    if not json_responses:
-        raise ValueError(
-            "Could not find JSON Response folder. Use the environment variable KODI_STUB_RPC_RESPONSES to set one")
+    if json_responses:
+        path = "{}.json".format(os.path.join(json_responses, json_data["method"].lower()))
+        if os.path.isfile(path):
+            with io.open(path, mode='r', encoding='utf-8') as fd:
+                stub_content = json.loads(fd.read())
+                if isinstance(stub_content, dict):
+                    return json.dumps(stub_content)
+                try:
+                    return json.dumps(next(stub.get('response')
+                                           for stub in stub_content
+                                           if json_data.get('params')
+                                           and stub.get('request', {}).get('method') == json_data.get('method')
+                                           and stub.get('request', {}).get('params') == json_data.get('params')))
+                except StopIteration:
+                    pass
+    else:
+        KodiStub.print_line("Warning: Could not find JSON Response folder. Use the environment variable KODI_STUB_RPC_RESPONSES to set one.", color=Colors.Red)
 
-    path = "{}.json".format(os.path.join(json_responses, json_data["method"].lower()))
-    if os.path.isfile(path):
-        with io.open(path, mode='r', encoding='utf-8') as fd:
-            stub_content = json.loads(fd.read())
-            if isinstance(stub_content, dict):
-                return json.dumps(stub_content)
-            try:
-                return json.dumps(next(stub.get('response')
-                                       for stub in stub_content
-                                       if json_data.get('params')
-                                       and stub.get('request', {}).get('method') == json_data.get('method')
-                                       and stub.get('request', {}).get('params') == json_data.get('params')))
-            except StopIteration:
-                pass
-
-    return '{"id":1,"jsonrpc":"2.0","result":"OK"}'
+    return json.dumps(dict(id=1, jsonrpc="2.0", result="OK"))
 
 
 # noinspection PyPep8Naming
@@ -746,8 +745,15 @@ def executebuiltin(function):
     See: http://kodi.wiki/view/List_of_Built_In_Functions
 
     """
+    from sakee.sakebuiltin import BuiltinApi
 
-    KodiStub.print_line("Executebuiltin: {0}".format(function), color=Colors.Blue)
+    try:
+        # Implement some methods for real
+        BuiltinApi().handle(function)
+
+    except NotImplementedError:
+        # Fallback to stubs
+        KodiStub.print_line("Executebuiltin: {0} is not implemented".format(function), color=Colors.Red)
 
 
 # noinspection PyPep8Naming
@@ -762,7 +768,29 @@ def getCondVisibility(condition):  # NOSONAR
     List of Conditions - http://kodi.wiki/view/List_of_Boolean_Conditions
 
     """
+
+    if "+" in condition:
+        final_result = 1
+        conditions = condition.split("+", 1)
+        for c in conditions:
+            sub_result = __get_cond_visibility(c)
+            final_result = final_result and sub_result
+
+    elif "|" in condition:
+        final_result = 0
+        conditions = condition.split("|", 1)
+        for c in conditions:
+            sub_result = __get_cond_visibility(c)
+            final_result = final_result or sub_result
+    else:
+        final_result = __get_cond_visibility(condition)
+
+    return final_result
+
+
+def __get_cond_visibility(condition):
     result = False
+    condition = condition.strip()
     if condition.startswith('System.HasAddon('):
         add_on_id = condition.replace('System.HasAddon(', '').replace(')', '').strip('"')
         add_ons = os.listdir(os.path.join(__add_on_info.kodi_home_path, "addons"))
@@ -773,6 +801,9 @@ def getCondVisibility(condition):  # NOSONAR
 
         result = add_on_id in add_ons
 
+    elif condition.startswith('System.AddonIsEnabled('):
+        result = True
+
     elif condition == 'system.platform.windows':
         result = os.name == "nt"
 
@@ -837,6 +868,20 @@ def getLanguage(format=ENGLISH_NAME, region=False):  # NOSONAR
     return "English"
 
 
+# noinspection PyShadowingBuiltins,PyPep8Naming
+def getLocalizedString(id):  # NOSONAR
+    """ Get a localized `unicode string`.
+
+    :param int id:  id# for string you want to localize.
+
+    :return: Localized `unicode string`
+    :rtype: str
+
+    """
+
+    return "xbmc:po:{}".format(id)
+
+
 def log(msg, level=0):
     """ Write a string to Kodi's log file and the debug window.
 
